/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005,2007  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#ifndef DEVICE_HXX_INCLUDED
#define DEVICE_HXX_INCLUDED 1

class UiDrawer;

class	Device
{
public:
	Device(std::string clazz = "", std::string name = "");
	virtual ~Device();
//
// Load settings from XML document
//
        bool parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr parent);
//
// Save settings to XML document
//
        bool store(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr parent);

	virtual bool isEnabled(void);
	virtual void enable(bool b = true);

	virtual void reset() = 0;
//
// Settings functions.
// See if this device has user-settable options. If it does, populates
// "key" and "caption" and returns true; else, returns false.
//
        virtual bool hasSettings(SDLKey *key, string &caption);
//
// Display settings screen
// Return 0 if OK, -1 if quit message received.
// 
        virtual UiEvent settings(UiDrawer *d);
        virtual bool parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) = 0;
        virtual bool storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) = 0;
        bool parseEnabled(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
        bool storeEnabled(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
        bool parseFilename(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, 
                           const string nodename, string &name);
        bool storeFilename(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, 
                           const string nodename, string name);
        bool parseColour(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
                           const string nodename, SDL_Colour &fg, SDL_Colour &bg
);
        bool storeColour(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
                           const string nodename, SDL_Colour &fg, SDL_Colour &bg
);
        bool parseInteger(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
                           const string nodename, int *i);
        bool storeInteger(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
                           const string nodename, int i);

private:
        void putColour  (xmlNodePtr cur, const string attrname, SDL_Colour &clr)
;
        void parseColour(xmlNodePtr node, const string attrname, SDL_Colour &clr
);

protected:
        string m_class, m_name;
private:
        bool m_enabled;
};

#endif	// ndef DEVICE_HXX_INCLUDED
