/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005-6  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


#include "qdae.hxx"
#include "qdae_ui.hxx"
#include "qdae_files.hxx"
#include "sdl_keyboard.hxx"
#include "f1_keyboard.hxx"
#include "rascal.hxx"
#include "machine_f1.hxx"

F1_Keyboard::F1_Keyboard(Rascal_ENV *m) : Keyboard((PC_ENV *)m, "F1")
{
	m_rascal = m;
}

F1_Keyboard::~F1_Keyboard()
{

}

/* Generate the "Here is the time" packet which the Rascal keyboard can be 
 * made to transmit */
void F1_Keyboard::send_time()
{
	unsigned short pkt[16];
	time_t t;
	struct tm *ptm;
	int n;

	time (&t);
	ptm = localtime(&t);
	pkt[ 0] = 0x013;	/* Time set */
	pkt[ 1] = 0x411;
	pkt[ 2] = 0x402 | ((ptm->tm_hour / 10) << 4);
	pkt[ 3] = 0x403 | ((ptm->tm_hour % 10) << 4);
	pkt[ 4] = 0x404 | ((ptm->tm_min / 10) << 4);
	pkt[ 5] = 0x405 | ((ptm->tm_min % 10) << 4);
	pkt[ 6] = 0x406 | ((ptm->tm_sec / 10) << 4);
	pkt[ 7] = 0x407 | ((ptm->tm_sec % 10) << 4);
	pkt[ 8] = 0x408 | ((ptm->tm_mday / 10) << 4);
	pkt[ 9] = 0x409 | ((ptm->tm_mday % 10) << 4);
	pkt[10] = 0x40a | ((ptm->tm_mon / 10) << 4);
	pkt[11] = 0x40b | ((ptm->tm_mon % 10) << 4);
	pkt[12] = 0x40c | (((ptm->tm_year % 100) / 10) << 4);
	pkt[13] = 0x40d | ((ptm->tm_year % 10) << 4);
	pkt[14] = 0x40e;
	pkt[15] = 0x40f;
	for (n = 0; n < 16; n++)
	{
		m_rascal->write_infrared(pkt[n]);
	}
//	Sys.debug = 3;
}


void F1_Keyboard::encode_mouse(unsigned mask, SDL_Event *e, signed rel)
{
	unsigned mouse;
	signed char delta;

	while (rel)
	{
		mouse = 0x800 | mask;
		if (rel < 0)
		{
			if (rel < -127) delta = -127;
			else delta = rel;
		}
		else
		{
			if (rel > 127) delta = 127;
			else delta = rel;
		}
		mouse |= (delta & 0xFF);
//		fprintf(stderr, "delta=%d rel=%d encoded=%x\n",
//				delta, rel, mouse);
		m_rascal->write_infrared(mouse);
		rel -= delta;
	}	
}

UiEvent F1_Keyboard::keyboard_event(SDL_Event *e)
{
	int n;
	unsigned shift = 0;
	unsigned ctrl  = 0;
	unsigned repeat= 0;
	static unsigned btn_state = 0;
	int raskey;

	switch(e->type)
	{
		case SDL_QUIT: return UIE_QUIT;

		case SDL_KEYDOWN:
			if (e->key.keysym.mod & KMOD_SHIFT) shift = 0x100;
			else				    shift = 0;
			if (e->key.keysym.mod & KMOD_CTRL)  ctrl  = 0x200;
			else				    ctrl  = 0;

/* Convert RightShift + F1-F6 to the MicroScreen keys. */
			if (e->key.keysym.mod & KMOD_RSHIFT &&
			    e->key.keysym.sym >= SDLK_F1 &&
			    e->key.keysym.sym <= SDLK_F6)
			{
/* Only if LeftShift is also held down do we get shifted */
				if (e->key.keysym.mod & KMOD_LSHIFT) 
					shift = 0x100;
				else	shift = 0x000;
				int *j = (int *)&e->key.keysym.sym;
				(*j) += (SDLK_F13 - SDLK_F1);
			}
			// XXX: Set 'repeat' if code generated by repeating key

			if (e->key.keysym.sym >= SDLK_FIRST && 
			    e->key.keysym.sym <= SDLK_LAST)
			{
				if ((e->key.keysym.mod & KMOD_RALT) ||
				    (e->key.keysym.mod & KMOD_MODE))
				  raskey = rascal_key_ralt[e->key.keysym.sym];
				else 
				  raskey = rascal_key_map[e->key.keysym.sym];
				if (raskey == -1) break;
				if (raskey == -2)
				{
					return gl_ui->mainMenu();		
				}
				if (raskey == 0x13)
				{
					send_time();
					break;
				}
				raskey &= 0xFF;
			       	m_rascal->write_infrared(raskey|shift|ctrl|repeat);
			}
			break;
		case SDL_MOUSEMOTION:
			encode_mouse(btn_state, e, e->motion.xrel);
			encode_mouse(btn_state | 0x100, e, e->motion.yrel);
			break;
		case SDL_MOUSEBUTTONDOWN:
			if (e->button.button == 1) btn_state |= 0x400;
			else			   btn_state |= 0x200;
/*			fprintf(stderr, "Encoding button %d as %x\n",
					e->button.button, btn_state); */
			m_rascal->write_infrared(0x800 | btn_state);
			break;
		case SDL_MOUSEBUTTONUP:
			if (e->button.button == 1) btn_state &= ~0x400;
			else			   btn_state &= ~0x200;
/*			fprintf(stderr, "Encoding button %d as %x\n",
					e->button.button, btn_state); */
			m_rascal->write_infrared(0x800 | btn_state);
			break;
	}
	return Keyboard::keyboard_event(e);
}


