/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2007  John Elliott <jce@seasip.demon.co.uk>

*************************************************************************/

// Based largely on e8253.c/h from pce by Hampa Hug. //
//////////////////////////////////////////////////////////////////////////////
// This program is free software. You can redistribute it and / or modify it /
// under the terms of the GNU General Public License version 2 as  published //
// by the Free Software Foundation.                                          //
//                                                                           //
// This program is distributed in the hope  that  it  will  be  useful,  but //
// WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General //
// Public License for more details.                                          //
///////////////////////////////////////////////////////////////////////////////


class PC_ENV;
class I8253;

#ifndef I8253_HXX_INCLUDED
#define I8253_HXX_INCLUDED

class TimerListener
{
	public:
	virtual void onTimerFire(I8253 *source, int channel, bool value) = 0;
};


class TimerChannel
{
	friend class I8253;

	short m_counter;
	u_int8 m_counterWrite;
	short m_outputLatch;
	u_int8 m_outputLatchRead;
	u_int8 m_counterRead;

	u_int8 m_sr;
	u_int8 m_rw;
	u_int8 m_mode;
	bool m_bcd;
	bool m_counting;
	bool m_gate;
	TimerListener *m_out;
	bool m_outVal;
	short m_val;
	I8253 *m_parent;
	int m_channel;

	TimerChannel(I8253 *par, int ch);
	void init();
	void setOut(bool val);

	void newVal0();
	void decVal0(int n);
	void setGate0(bool val);
	void newVal2();
	void decVal2(int n);
	void setGate2(bool v);
	void newVal3();
	void decVal3(int n);
	void setGate3(bool v);
	void newVal4();
	void decVal4(int n);
	void setGate4(bool v);

	void newVal();
	void decVal(int n);
	u_int8 getValue();
	void setValue(u_int8 val);
	void setGate(bool value);
	void latch();
	inline bool isCounting() { return m_counting; }
	inline void setTimerListener(TimerListener *t) { m_out = t; }
	void setControl(u_int8 val);
};

#define I8253_CHANNELS 3

class I8253 : public Device
{
	float stepSpeed, cpuSpeed;
	float floatSteps;

	TimerChannel *m_channel[I8253_CHANNELS];
public:
	I8253(PC_ENV *host);
	~I8253();

	void write(u_int16 port, u_int8 value);
	u_int8 read(u_int16 port);

	void tick();
	void setListener(int channel, TimerListener *tl);
	void setGate(int channel, bool v);

	virtual void reset();
	virtual bool parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	virtual bool storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);

private:
	PC_ENV *m_host;
};
#endif //ndef I8253_HXX_INCLUDED

