/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005,2008  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "qdae.hxx"
#include "qdae_files.hxx"
#include "qdae_ui.hxx"
#include "sdl_video.hxx"
#include "fp_video.hxx"
#include "rascal.hxx"
#include "machine_fp.hxx"

#include "sdl_keyboard.hxx"
#include "f1_keyboard.hxx"
#include "memsize.hxx"

#define XMLNS             "http://www.seasip.demon.co.uk/qdae_fp.dtd"
#define HW_FILE           "portable.xml" 

/* Created FP environment as a straight copy of F1 for now. */

FP_ENV::FP_ENV() :
	Rascal_ENV(HW_FILE, XMLNS),
	m_pit(this),
	m_pic(this),
	m_sio(this),
	m_fdc(this),
	m_winch(this)
{
	m_ramSelect = 0;
	m_keyboard = NULL;
	m_video = NULL;
}

FP_ENV::~FP_ENV()
{
	if (m_keyboard) delete m_keyboard;
	if (m_video) delete m_video;
}

PC_ENV *FP_ENV::clone()
{
	FP_ENV *other = new FP_ENV();
	copy_to(other);
	other->m_pit = m_pit;
	other->m_fdc = m_fdc;
	other->m_winch = m_winch;
	other->m_ramSelect = m_ramSelect;
	other->m_pic = m_pic;
	return other;
}

void FP_ENV::initialize()
{
	if (!m_keyboard) m_keyboard = new F1_Keyboard(this);
	if (!m_video)    m_video    = new FP_Video(this);

	addDevice(&m_sio);
	addDevice(&m_fdc);
	addDevice(&m_pic);
	addDevice(&m_pit);
	addDevice(&m_winch);
	addDevice(m_keyboard);
	addDevice(m_video);
	loadHardware();
	parseHardware();

	PC_ENV::initialize();

/* Put the SIO on IRQ4. Other interrupts:
 * 	IRQ0: Timer
 * 	IRQ1: FDC
 * 	IRQ2: Voice chip
 * 	IRQ6: Parallel port
 * 	IRQ7: DMA controller */
	m_sio.setPIC(&m_pic, 4);
	load_rom("fp.rom", 0xF8000, 0x8000, "portable.rom");
}



void FP_ENV::sys_out8(u_int16 port, u_int8 value)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port - 0x1E0, value);
		return;
	}
	switch(port)
	{
		case 0x00:
		case 0x02:
		case 0x04:
		case 0x06:
			m_fdc.write((port & 6) / 2, value);
			break;
		case 0x08:
		case 0x0A:
		case 0x0C:
		case 0x0E:
			m_pit.write((port & 6) / 2, value);
			break;
		case 0x18:
		case 0x1A:
		case 0x1C:
		case 0x1E:
			m_sio.write((port & 6) / 2, value);
			break;
		case 0x26:	/* XXX sound generator */
			break;
		case 0x28:	/* LCD contrast */
		case 0x6C:	/* CRTC register select */
		case 0x6E:	/* CRTC register write */
			((FP_Video *)m_video)->write(port, value);
			break;
		case 0x2E:
			m_ramSelect = value;
			((FP_Video *)m_video)->write(port, value);
			break;
		case 0x30:	/* Drive select, 1=A: 2=B: */
			m_fdc.select_drive((~value) & 1);
			break;
		case 0x32:
			m_fdc.load_head(value != 0);
			break;
		case 0x68:
		case 0x6A:
			m_pic.write((port / 2) & 1, value);
			break;
/* Below this line, F1 ports */
		case 0x01:
			break;
		case 0x05:
			m_fdc.set_motor(value);
			break;
		case 0x07:
		case 0x09:
		case 0x0B:
		case 0x0D:
			m_video->write(port,value);
			break;
		case 0x0F:
/* XXX Implement! 	printer_write(port, value); */
			break;	
		default:
			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%02x\n", 
					R_CS, R_IP, port, value); 
			break;
	}
}


void FP_ENV::sys_out16(u_int16 port, u_int16 value)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		m_winch.write(port, value & 0xFF);
		return;
	}
	switch(port)
	{
		case 0x08:
		case 0x0A:
		case 0x0C:
		case 0x0E:
		case 0x2E:
			m_ramSelect = value;
			((FP_Video *)m_video)->write16(port, value);
			break;
		case 0x18:
		case 0x1A:
		case 0x1C:
		case 0x1E:
		case 0x30:
		case 0x26:
		case 0x68:	/* PIC */
		case 0x6A:	/* PIC */
		case 0x6C:	/* CRTC register select */
		case 0x6E:	/* CRTC register write */
/* XXX F1 ports */
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x20:
		case 0x22:
		case 0x24:
		case 0x01:
		case 0x05:
		case 0x40:
		case 0x42:
		case 0x44:
		case 0x46:
			sys_out8(port, value & 0xFF);
			break;
		default:
/*			fprintf(stderr, "Unknown OUT at %04x:%04x: "
					"port=%04x value=%04x\n", 
					m->R_CS, m->R_IP, port, value); */
			break;
	}
}

u_int8 FP_ENV::sys_in8(u_int16 port)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	}
	switch(port)
	{
		case 0x00:
		case 0x02:
		case 0x04:
		case 0x06:
			return m_fdc.read((port & 6) / 2);
		case 0x08:
		case 0x0A:
		case 0x0C:
		case 0x0E:
			return m_pit.read((port & 6) / 2);
		case 0x18:
		case 0x1A:
		case 0x1C:
		case 0x1E:
			return m_sio.read((port & 6) / 2, false);
		case 0x68:
		case 0x6A:
			return m_pic.read((port / 2) & 1);
		case 0x6C:
		case 0x6E:
			return ((FP_Video *)m_video)->read(port);	
		default:
/*			fprintf(stderr, "Unknown IN8: port=%04x\n", port); */
			break;
	}
	return 0xFF;
}

u_int16 FP_ENV::sys_in16(u_int16 port)
{
	if (port >= 0x1E0 && port <= 0x1FF)
	{
		return m_winch.read(port - 0x1E0);
	}
	switch(port)
	{
		case 0x00:
		case 0x02:
		case 0x04:
		case 0x06:
		case 0x10:
		case 0x12:
		case 0x14:
		case 0x16:
		case 0x18:
		case 0x1A:
		case 0x1C:
		case 0x1E:
		case 0x68:
		case 0x6A:
		case 0x6C:
		case 0x6E:
			return sys_in8(port);
		default:
/*			fprintf(stderr, "Unknown IN8: port=%04x\n", port); */
			break;
	}
	return 0xFFFF;
}





void FP_ENV::clock_tick()
{
	PC_ENV::clock_tick();

	m_pit.tick();
	m_sio.tick();
	m_pic.tick();
/* XXX Speed regulation */
}



void FP_ENV::reboot()
{
	m_ramSelect = 0;
	m_pit.reset();
	m_sio.reset();
	m_fdc.reset();
	m_winch.reset();
	/* XXX Add resets for other peripherals */
	PC_ENV::reboot();

	/* XXX Enable timer channels & set listeners */
}


UiEvent FP_ENV::floppy_insert(int drive, std::string filename, const char *type)
{
	m_fdc.insert(drive, filename, type);
}

void FP_ENV::floppy_eject(int drive)
{
	m_fdc.eject(drive);
}


void FP_ENV::write_infrared(u_int16 value)
{
	m_sio.write_ir(0, value);
}

void FP_ENV::poll_events()
{
	if (gl_f10count)
	{
		--gl_f10count;
		if (!gl_f10count) m_keyboard->send_time();
	}
	PC_ENV::poll_events();
}


u_int8 FP_ENV::fetch(u_int32 offset)
{
/* If VRAM paged in, it comes over the top of normal RAM */
	if ((m_ramSelect & 0x80) && offset >= 0xD0000 && offset < 0xF0000)
	{
		return ((FP_Video *)m_video)->fetch(offset - 0xD0000);
	}
	if (offset >= m_memsize->m_memsize && offset < 0xF8000) return 0xFF;
	return m_mem_base[offset];
}

void FP_ENV::store(u_int32 offset, u_int8 value)
{
/* If VRAM paged in, it comes over the top of normal RAM */
	if ((m_ramSelect & 0x80) && offset >= 0xD0000 && offset < 0xF0000)
	{
		((FP_Video *)m_video)->store(offset - 0xD0000, value);
		return;
	}
	if (offset < m_memsize->m_memsize) m_mem_base[offset] = value;
}


void FP_ENV::sys_lock()
{
}

