
/* 
 * Dos/PC Emulator
 * Copyright (C) 1991 Jim Hudgens
 * 
 * 
 * The file is part of GDE.
 * 
 * GDE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * GDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GDE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 *
 */

static char rcsid[]=
  "$Id: $";

/* $Log: $
 * Revision 0.02  1991/07/30  02:07:09  hudgens
 * added copyright
 *
 * Revision 0.01  1991/07/20  04:10:44  hudgens
 * Initial revision
 *
 */



#define MAIN
#include "qdae.hxx"
#include "qdae_files.hxx"
#include "qdae_ui.hxx"
#include "sdl_video.hxx"
#include "colours.hxx"

#include "machine_xi.hxx"
#include "rascal.hxx"
#include "machine_f1.hxx"
#include "machine_fp.hxx"

IS_VOLATILE int intr;

extern string *gl_homeRoot, *gl_keyboard;

PC_ENV  *gl_Sys;

#if defined HAVE_STRCASECMP
#define scmp strcasecmp
#elif defined HAVE_STRICMP
#define scmp stricmp
#elif defined HAVE_STRCMPI
#define scmp strcmpi
#else
#define scmp strcmp
#endif

SYSTEMTYPE machine_type(const char *s)
{
	if (!scmp(s, "pc") || !scmp(s, "xi")) return PCSERIES;
	if (!scmp(s, "fp") || !scmp(s, "portable")) return RASCALA;
	if (!scmp(s, "f1") || !scmp(s, "f2") || !scmp(s, "f10")) return RASCALC;
	if (!scmp(s, "xen")) return XEN;	

	fprintf(stderr, "Unrecognised system type: %s\n", s);
	exit(1);
}

void detect_machine(const char *filename, SYSTEMTYPE *type)
{
	DSK_PDRIVER dskfile;
	DSK_GEOMETRY dg;
	dsk_err_t err;
	unsigned char buf[512];

	err = dsk_open(&dskfile, filename, NULL, NULL);
	if (err) return;
	/* Select PCW/IBM 180k format, which is sane enough for our purposes */
	err = dg_stdformat(&dg, FMT_180K, NULL, NULL);
	/* Load the boot sector */
	if (!err) err = dsk_lread(dskfile, &dg, buf, 0);
	/* Check that it is an Apricot boot sector */
	if (!err) err = dg_aprigeom(&dg, buf);
	dsk_close(&dskfile);
	if (err) return;
	/* OK, we have an Apricot-format disk. Let's see what boot type it
	 * is */
	switch(buf[11])
	{
		case 1: *type = PCSERIES; 
			printf("Hardware type autodetected: PC/Xi\n");
			break;
		case 2: if (*type != RASCALA && *type != RASCALC)
			{
				*type = RASCALC; 
				printf("Hardware type autodetected: F-series\n");
			}
			break;
/* XXX others? */
	}
}






/*
void ihandle(int)
{
	printf("handled\n");
	exit(0);
}
 */

int main(int argc, char **argv)
{
	int dashdash;
	int drive;
	string fbuf;
	int n, arg;
	int machine_forced = 0;

	/* new these, because having them constructed globally upsets the 
	 * gcc I use for cross-compiling. */
	gl_homeRoot = new std::string();
	gl_keyboard = new std::string();

	GeneralSettings init_settings;

	for (n = 0; n < MAX_FLOPPIES; n++)
	{
		floppy_filename[n] = new std::string();
	}

/* In QDAE, don't bother catching signals. 
	signal(1, ihandle);
	signal(2, ihandle);
	signal(3, ihandle); */

/* GDE argument parsing removed. Argument parsing in QDAE is 
 * nearly all about options, and these get parsed in sys_config_args().
 */
	
/* Configure QDAE. Look first for a .qdaerc in the current 
   directory, then in the user's HOME dir.  If either of these
   fail, fall back to minimalist defaults. */

/* For 0.0.2, use the utility function getHomeDir() which works under
 * Windows and can read /etc/passwd if $HOME is not set.
 */

	string homeDir = getHomeDir();
	fbuf = homeDir + "/" RCFILE;

	*gl_homeRoot = DEF_HOMEDIR;
	*gl_keyboard = DEF_KEYBOARD;
	if ( access(RCFILE,R_OK) == 0)		 init_settings.loadConfig(RCFILE);
	else if (access(fbuf.c_str(),R_OK) == 0) init_settings.loadConfig(fbuf);
	else					 init_settings.defaults();

/* Argument parsing not involving options. The first filename becomes
 * drive A:, the second drive B:. */
	dashdash = 0;
	drive = 0;
	for (arg = 1; arg < argc; arg++)
	{
		if (!strcmp(argv[arg], "--"))
		{
			dashdash = 1;
			continue;
		}
		if (argv[arg][0] == '-' && !dashdash) 
		{
			if (argv[arg][1] == 'm' || argv[arg][1] == 'M')
			{
				if (argv[arg][2] == 0)
				{
					++arg;
					if (arg >= argc)
					{
						fprintf(stderr, "%s: -m option must be followed by a machine type.\n");
						return 1;
					}
					init_settings.type = machine_type(argv[arg]);
					machine_forced = 1;
				}
				else	
				{
					init_settings.type = machine_type(argv[arg]+2);
					machine_forced = 1;
				}
			}
			continue;
		}
		if (drive > 1) syntax(argv);
		*floppy_filename[drive++] = argv[arg];
/* If we're starting with a floppy in the drive, wait for 80 interrupts and 
 * then simulate someone pressing Time/Date */
		gl_f10count = 80;
	}
/* If no machine type is specified on the command line, and we have a 
 * boot image, get machine type from that. Note that this cannot distinguish
 * between boot images for the Rascal A (portable) and Rascal C (F-series) */
	if (floppy_filename[0][0] != "" && !machine_forced)
	{
		detect_machine(floppy_filename[0]->c_str(), &init_settings.type);
	}

	switch(init_settings.type)
	{
		case PCSERIES:
			gl_Sys = new XI_ENV();
			break;
		case RASCALA:
			gl_Sys = new FP_ENV();
			break;
		case RASCALC:
			gl_Sys = new F1_ENV();
			break;

		default:
			gl_Sys = new F1_ENV();
			//gl_Sys = new PC_ENV();
			break;
	}
	gl_Sys->settings = init_settings;
	gl_Sys->initialize();
	gl_Sys->reboot();

	gl_Sys->i86_exec();
  
	delete gl_Sys;	
	delete gl_homeRoot;
	delete gl_keyboard;

	for (n = 0; n < MAX_FLOPPIES; n++)
	{
		delete floppy_filename[n];
	}
	return 0; 
}



void syntax(char **argv)
{
	fprintf(stderr, "Usage: %s { options } { a_disk { b_disk }}\n",
			argv[0]);
	fprintf(stderr, "Options are:\n"
			"-h homedir   Select home directory (default is %s)\n"
			"-k keyboard  Select keyboard layout (default is %s)\n"
			"-m machine   PC or F1\n" 
			, DEF_HOMEDIR, DEF_KEYBOARD);
	exit(1);
}
