/* 
 * Dos/PC Emulator
 * Copyright (C) 1991 Jim Hudgens
 * Copyright 2005,2007 John Elliott 
 * 
 * The file is part of GDE.
 * 
 * GDE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * GDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GDE; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 *
 */

#ifndef QDAE_H_INCLUDED
#define QDAE_H_INCLUDED

#define QDAE_VERSION "0.0.4"

static char gde_h_rcsid[]=
  "$Id: $";


/* $Log: $
 *
 */

#include "ansidecl.h"
#include "sysdep.h"

/*
#define FETCH_BYTE(to,from,offset) to = from[offset];
#define FETCH_WORD(to,from,offset) to=from[offset]&0xff; \
  to|=(from[offset+1]<<8); 
#define STORE_BYTE(to,offset,from)   to[offset] = (from);
#define STORE_WORD(to,offset,from)   to[offset] = (from)&0xff;\
    to[offset+1] = ((from)>>8)&0xff;
*/

/* 8086 support structs and definitions */
#include "i86.hxx"
/* 8087 support structs and definitions. */
#include "i87.h"

#include "device.hxx"

typedef enum
{
	DEFAULT,
	PCSERIES,
	RASCALA,
	RASCALC,
	XEN	
} SYSTEMTYPE;


class GeneralSettings : public Device
{
	xmlDocPtr m_settings;
	string m_filename;

	string storeDebug();
	void parseDebug(const char *p);
	void parseConfig();

public:
	GeneralSettings();
	~GeneralSettings();

	void defaults();

        virtual void reset();
        virtual bool parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
        virtual bool storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
        virtual UiEvent settings(UiDrawer *d);
        virtual bool hasSettings(SDLKey *key, string &caption);

	bool loadConfig(const std::string file);
	bool saveConfig();

	void config(const std::string file);
	void configArgs(int argc, char **argv);

	int		debug;
	SYSTEMTYPE	type;
};

#include "qdae_system.hxx"

/* GLOBAL */
extern IS_VOLATILE int intr;

#include "i86_externs.h"
#include "i87_externs.h"
#include "sys_debug.h"
//#include "apricot.h"
#include "video_externs.hxx"
#include "keyb_externs.hxx"

void syntax(void);
/* VARARGS subroutines */
void	sys_fatal(int err, char *fmt, ...);
void    sys_warn(char *fmt, ...);


void syntax(char **argv);
extern SYSTEMTYPE machine_type(const char *s);


#define DEF_KEYBOARD "uk"
#define DEF_HOMEDIR  ".qdae"
#define RCFILE	     ".qdaerc"

#ifdef MAIN
#define EXT 
#define EQ(x) = x
#else
#define EXT extern
#define EQ(x)
#endif

     
#endif

