/************************************************************************

    QDAE v0.0.2 - Quick And Dirty Apricot emulator

    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#include "UiControl.hxx"

UiEvent keyboard_event(SDL_Event *e);

class UiMenuBar;
class SdlTerm;

class QDAE_UI : public UiDrawer
{
public:
	QDAE_UI(PC_ENV *host, SDL_Surface *s);
	~QDAE_UI();

	UiEvent report(const string s, const string prompt);
	UiEvent reportV(const string s, const char *prompt, ...);

	inline bool usingScreen() { return m_nUsingScreen != 0; }

/* UiDrawer overrides */
	virtual void measureString(const string &s, Uint16 *w, Uint16 *h);
	virtual void measureMenuBorder(SDL_Rect &rcInner, SDL_Rect &rcOuter);
	virtual void measureMenuBar(SDL_Rect &rc);

	virtual void drawRect   (int selected, SDL_Rect &rc, SDL_Colour *c = NULL);
	virtual void drawString (int selected, Uint16 x, Uint16 y, const std::string s);
	virtual void drawPicture(int selected, Uint16 x, Uint16 y, SDL_Surface *s);
	virtual void drawGlyph  (int selected, Uint16 x, Uint16 y, UiGlyph g);
	virtual void drawMenuBorder(SDL_Rect &rc, SDL_Rect &rcInner);
        virtual void drawMenuBar   (int selected, SDL_Rect &rc);
        virtual void drawSeparator (int selected, SDL_Rect &rc);
	virtual void drawCursor    (Uint16 x, Uint16 y, char ch);

	virtual bool getManualUpdate(void);
	virtual void setManualUpdate(bool b = true);
	virtual void updateRect(SDL_Rect &rc);

        virtual SDL_Surface *saveSurface(SDL_Rect &rc);
        virtual void restoreSurface(SDL_Surface *s, SDL_Rect &rc);
	virtual void centreContainer(UiContainer &c);

        virtual UiEvent menuEvent(SDL_Event &e); 
        virtual char getKey(void);
	virtual void onTimer50(void);

	UiEvent mainMenu(void);
	UiEvent illegal_op(void);
	UiEvent halted(void);
	UiEvent i8089_error(const char *message);

protected:
	SDL_Surface *m_surface;
	SdlTerm *m_term;
	int m_nUsingScreen;
	SDL_Colour m_clrFG, m_clrBG;
	UiMenuBar *m_menuBar;
	PC_ENV *m_host;

	void acquireScreen();
	void releaseScreen();
	void menuDraw();

	UiEvent menuF1(int x, int y);
	UiEvent menuF2(int x, int y);
	UiEvent menuF3(int x, int y);
	UiEvent menuF6(int x, int y);
	UiEvent menuF7(int x, int y);
	UiEvent registers(void);
	UiEvent aboutBox(void);
	UiEvent confirm(int x, int y, const string prompt);
	UiEvent requestDisc(string &filename, const char **type);
	UiEvent askCreateDisc(string &filename, int fmt, const char **type);
	UiEvent getDiscType   (int &fmt, bool create);
	void showRegs(char buf[4][80]);
};

#ifdef MAIN
QDAE_UI *gl_ui;
#else
extern QDAE_UI *gl_ui;
#endif

