/************************************************************************* 
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005-7  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


#include "qdae.hxx"
#include "qdae_ui.hxx"
#include "qdae_files.hxx"
#include "sdl_keyboard.hxx"

string *gl_keyboard;

Keyboard::Keyboard(PC_ENV *m, std::string name) : Device("keyboard", name)
{
	string kbName;
	FILE *fp;
	char buf[255];
	int pckey, raskey, xenkey, sdlkey;

	host = m;
	kbName = *gl_keyboard;
	kbName += ".kbd";

	for (int n = SDLK_FIRST; n <= SDLK_LAST; n++)
	{
		pcxi_key_map[n] = -1;
		rascal_key_map[n] = -1;
		xen_key_map[n] = -1;
		pcxi_key_ralt[n] = -1;
		rascal_key_ralt[n] = -1;
		xen_key_ralt[n] = -1;
	}
	fp = openQdaeFile(FT_KEYBOARD, kbName, "r");
	if (!fp)
	{
		fprintf(stderr, "Could not open keyboard %s, trying %s\n",
				kbName.c_str(), DEF_KEYBOARD ".kbd");
		fp = openQdaeFile(FT_KEYBOARD, DEF_KEYBOARD ".kbd", "r");
	}
	if (!fp)
	{
		string err = "Could not load keyboard: " + kbName;
                gl_ui->report(err, "  Exit  "); 
		exit(1);
	}
	while (fgets(buf, sizeof(buf), fp))
	{
		char *hash = strchr(buf, '#');
		char *ptr = buf;
		SDLMod mod;

		if (hash) *hash = 0;
		pckey  = get_number(&ptr, -1);
		raskey = get_number(&ptr, -1);
		xenkey = get_number(&ptr, -1);
		do
		{
			mod = KMOD_NONE;
			sdlkey = get_number(&ptr, 0, 1, &mod);
			if (sdlkey)
			{
/* If right-ALT pressed or AltGr pressed: */
				if ((mod & KMOD_RALT) || (mod & KMOD_MODE))
				{
					if (pckey != -1)  
					  setkey(pcxi_key_ralt,  pckey, sdlkey);
					if (raskey != -1) 
					  setkey(rascal_key_ralt,raskey,sdlkey);
					if (xenkey != -1) 
					  setkey(xen_key_ralt,   xenkey,sdlkey);
				}
				else
				{
					if (pckey != -1)  
					  setkey(pcxi_key_map,   pckey, sdlkey);
					if (raskey != -1) 
					  setkey(rascal_key_map, raskey,sdlkey);
					if (xenkey != -1) 
					  setkey(xen_key_map,    xenkey,sdlkey);
				}
			}
		} while (sdlkey);	
	}	
}


Keyboard::~Keyboard()
{

}


void Keyboard::reset()
{

}


bool Keyboard::parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *cur)
{
	/* XXX Load layout here based on XML settings */
	return true;
}

bool Keyboard::storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *cur)
{
	return true;
}



void Keyboard::setkey(int *map, int pckey, int sdlkey)
{
	if (!sdlkey) return;
	if (sdlkey < SDLK_FIRST || sdlkey >= SDLK_LAST)
	{
		fprintf(stderr, "Warning: SDL key number %d out of range\n",
				sdlkey);
	}
	if (map[sdlkey] != -1)
	{
		fprintf(stderr, "Warning: Assigning to SDL key %d which is" 
				" already assigned.\n", sdlkey);

	}
	map[sdlkey] = pckey;
}

int Keyboard::get_number(char **ptr, int def, int star, SDLMod *mod)
{
	int d;

	while (isspace(**ptr)) ++(*ptr);

/* If key token preceded by '*', make it set right-alt modifier */
	if (star && (**ptr) == '*')
	{
		if (mod) *mod = KMOD_RALT;
		++(*ptr);	
	}

	if ((**ptr) != '-' && !isdigit(**ptr)) return def;
	d = atoi(*ptr);
	while (isdigit(**ptr) || (**ptr) == '-') ++(*ptr);
	return d;
}


UiEvent Keyboard::keyboard_event(SDL_Event *e)
{
	return UIE_CONTINUE;
}

void Keyboard::send_time()
{

}

