/*************************************************************************
 
    Quick And Dirty Apricot Emulator
    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/


#ifndef Z80_SIO_HXX
#define Z80_SIO_HXX

class PC_ENV;

class I8259;

struct SerialListener;

struct SerialChannel
{
	u_int8 number;
	u_int8 reg[8];
	u_int reg_sel;
	u_int8 tx_queue[512];
	u_int8 rx_queue[512];
	u_int tx_qhead, tx_qtail;
	u_int rx_qhead, rx_qtail;
	int rx_interrupt;
	int isr_countdown;
	SerialListener *m_listener;

	void command(int cmd);
	void reset();
	void write_rx(u_int8 value);
	void write_tx(u_int8 value);
	u_int8 read_rx();
	u_int8 read_tx();
};

struct SerialListener
{
	virtual void receive(SerialChannel *ch) = 0;
};


class Z80_SIO : public Device, public IntAck
{
public:
	Z80_SIO(PC_ENV *host);
	~Z80_SIO();

	void opcode(u_int8 value);
	void write(u_int16 port, u_int8 value);
	u_int8 read(u_int16 port, bool locked);
	u_int8 tick(void);

	virtual u_int8 ackInterrupt(void);
/*
	void raise(int channel); */

	inline void setPIC(I8259 *pic, int picirq)
	{
		m_pic = pic;
		m_picirq = picirq;
	}
	void setListener(int channel, SerialListener *l);

	void write_ir(int ch, u_int16 irword);
	void write_rx(int ch, u_int8 value);

	virtual void reset();
	virtual bool parseNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	virtual bool storeNode(xmlDoc *doc, xmlNs *ns, xmlNode *node);
	void iret();

private:
	bool	m_raising;
	I8259 *m_pic;
	int m_picirq;
	int m_lastirq;
	u_int8 last_z80_op;
	u_int8 irq_base;
	SerialChannel sio_ch[2];
	PC_ENV *m_host;
};

#endif	// ndef Z80_SIO_HXX
